@@ -10,6 +10,7 @@ from singer import metadata
 from singer import utils
 from singer.metrics import Point
 from tap_zendesk import metrics as zendesk_metrics
+from tap_zendesk.sync import process_record
 
 
 LOGGER = singer.get_logger()
@@ -32,6 +33,16 @@ def get_sideload_objects(stream):
     """Returns the value of sideload-objects from metadata, returns None if no values are present"""
     return metadata.to_map(stream.metadata).get((), {}).get('sideload-objects')
 
+def check_end_date(record, config, replication_key):
+    if 'end_date' in config:
+        record_key_date = process_record(record)[replication_key]
+        end_date = round(datetime.datetime.strptime(config['end_date'], "%Y-%m-%dT%H:%M:%SZ").timestamp())
+        if not isinstance(record_key_date, int):
+            record_key_date = round(datetime.datetime.strptime(record_key_date, "%Y-%m-%dT%H:%M:%SZ").timestamp())
+        return record_key_date > end_date
+    else:
+        return False
+
 def get_abs_path(path):
     return os.path.join(os.path.dirname(os.path.realpath(__file__)), path)
 
@@ -145,6 +156,8 @@ class Organizations(Stream):
         bookmark = self.get_bookmark(state)
         organizations = self.client.organizations.incremental(start_time=bookmark)
         for organization in organizations:
+            if check_end_date(organization, self.config, self.replication_key):
+                break
             self.update_bookmark(state, organization.updated_at)
             yield (self.stream, organization)
 
@@ -171,7 +184,10 @@ class Users(Stream):
         bookmark = self.get_bookmark(state)
         start = bookmark - datetime.timedelta(seconds=1)
         end = start + datetime.timedelta(seconds=search_window_size)
-        sync_end = singer.utils.now() - datetime.timedelta(minutes=1)
+        if 'end_date' in self.config:
+            sync_end = datetime.datetime.strptime(self.config['end_date'], "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=pytz.UTC)
+        else:
+            sync_end = singer.utils.now() - datetime.timedelta(minutes=1)
         parsed_sync_end = singer.strftime(sync_end, "%Y-%m-%dT%H:%M:%SZ")
 
         # ASSUMPTION: updated_at value always comes back in utc
@@ -274,6 +290,10 @@ class Tickets(Stream):
         for ticket in tickets:
             zendesk_metrics.capture('ticket')
             generated_timestamp_dt = datetime.datetime.utcfromtimestamp(ticket.generated_timestamp).replace(tzinfo=pytz.UTC)
+
+            if check_end_date(ticket, self.config, self.replication_key):
+                break
+
             self.update_bookmark(state, utils.strftime(generated_timestamp_dt))
 
             ticket_dict = ticket.to_dict()
@@ -375,7 +395,10 @@ class SatisfactionRatings(Stream):
         #start = bookmark_epoch-1
         start = bookmark - datetime.timedelta(seconds=1)
         end = start + datetime.timedelta(seconds=search_window_size)
-        sync_end = singer.utils.now() - datetime.timedelta(minutes=1)
+        if 'end_date' in self.config:
+            sync_end = datetime.datetime.strptime(self.config['end_date'], "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=pytz.UTC)
+        else:
+            sync_end = singer.utils.now() - datetime.timedelta(minutes=1)
         epoch_sync_end = int(sync_end.strftime('%s'))
         parsed_sync_end = singer.strftime(sync_end, "%Y-%m-%dT%H:%M:%SZ")
 
@@ -539,6 +562,8 @@ class TicketMetricEvents(Stream):
 
         ticket_metric_events = self.client.ticket_metric_events(start_time=bookmark)
         for ticket_metric_event in ticket_metric_events:
+            if check_end_date(ticket_metric_event, self.config, self.replication_key):
+                break
             if utils.strptime_with_tz(ticket_metric_event.time) >= bookmark:
                 # NB: We don't trust that the records come back ordered by
                 # updated_at (we've observed out-of-order records),
@@ -546,6 +571,20 @@ class TicketMetricEvents(Stream):
                 self.update_bookmark(state, ticket_metric_event.time)
                 yield (self.stream, ticket_metric_event)
 
+class AgentsActivity(Stream):
+    name = "agents_activity"
+    replication_method = "FULL_TABLE"
+
+    def sync(self, state): # pylint: disable=unused-argument
+
+        page = 1
+        agents_activity = self.client.talk.agents_activity(page=page)
+        while len(agents_activity) > 1:
+            for agent_activity in agents_activity:
+                yield (self.stream, agent_activity)
+            page = page + 1
+            agents_activity = self.client.talk.agents_activity(page=page)
+
 STREAMS = {
     "tickets": Tickets,
     "groups": Groups,
@@ -562,4 +601,5 @@ STREAMS = {
     "ticket_metrics": TicketMetrics,
     "sla_policies": SLAPolicies,
     "ticket_metric_events": TicketMetricEvents,
+    "agents_activity": AgentsActivity
 }
